#!/usr/bin/env python

"""
usage: test05.py [-h] [-v] code

synopsis:
  An FSM implemented with transistions module.

positional arguments:
  code           The code that must be entered to unlock

optional arguments:
  -h, --help     show this help message and exit
  -v, --verbose  Print messages during actions.

examples:
  python test05.py "abc"
"""


from __future__ import print_function
# import sys
from six.moves import input
import argparse
from transitions import Machine
#from transitions import State
#from transitions import Transition


def dbg_msg(verbose, msg):
    """Print a message if verbose is on."""
    if verbose:
        print(msg)


class CodeLock:
    """The model for the CodeLock FSM.

    Contains the call-back functions for the FSM.
    Carries state data.
    """

    def __init__(self, code, verbose):
        self.code = code
        self.verbose = verbose
        self.characters_entered = ''

    def add_entry(self, entry):
        dbg_msg(self.verbose, 'adding entry: {}'.format(entry))
        self.characters_entered += entry
        self.characters_entered = self.characters_entered[-len(self.code):]

    def code_matches(self, *args):
        dbg_msg(self.verbose,
                '(code_matches))  code: "{}"  entered: "{}"'.format(
                    self.code, self.characters_entered))
        if self.code == self.characters_entered:
            return True
        else:
            return False

    def code_does_not_match(self, *args):
        dbg_msg(self.verbose,
                '(code_does_not_match))  code: "{}"  entered: "{}"'.format(
                    self.code, self.characters_entered))
        if self.code != self.characters_entered:
            return True
        else:
            return False

    def show_state(self, *args):
        print('current state: {}'.format(self.state))


def set_up(opts):
    states = [
        {'name': 'locked', 'on_enter': 'show_state', },
        {'name': 'unlocked', 'on_enter': 'show_state', },
    ]
    transitions = [
        {
            'trigger': 'button',
            'source': 'locked',
            'dest': 'locked',
            'conditions': 'code_does_not_match',
            'before': 'add_entry',
        },
        {
            'trigger': 'try_handle',
            'source': 'locked',
            'dest': 'locked',
            'conditions': 'code_does_not_match',
        },
        {
            'trigger': 'try_handle',
            'source': 'locked',
            'dest': 'unlocked',
            'conditions': 'code_matches',
        },
        {
            'trigger': 'try_handle',
            'source': 'unlocked',
            'dest': 'unlocked',
        },
    ]
    # Make the model.
    code_lock = CodeLock(opts.code, opts.verbose)
    # Make the machine, adding the model.
    machine = Machine(
        model=code_lock,
        states=states,
        transitions=transitions,
        initial='locked',
    )
    return machine, code_lock


#
# Provide a test harness that can be called from the interactive prompt.
def make_machine(code='abc'):
    opts = argparse.Namespace(code=code, verbose=True)
    machine, model = set_up(opts)
    return machine, model


def main():
    description = """\
synopsis:
  An FSM implemented with transistions module.
"""
    epilog = """\
examples:
  python test05.py "abc"
"""
    parser = argparse.ArgumentParser(
        description=description,
        epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "code",
        type=str, default="abc",
        help="The code that must be used to unlock"
    )
    parser.add_argument(
        "-v", "--verbose",
        action="store_true",
        help="Print messages during actions.",
    )
    options = parser.parse_args()
    machine, model = set_up(options)
    while True:
        entry = input('enter char ("quit" to quit, "try" to check state): ')
        if entry == 'quit':
            break
        if entry == 'try':
            model.try_handle()
        else:
            model.button(entry)


if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    #import ipdb; ipdb.set_trace()
    main()
